
/* darkstat: a network traffic analyzer
 * (c) 2001-2002, Emil Mikulic.
 */

#include "spylog.h"

FILE *spylog = NULL;
time_t next_rotate = 0;

void spylog_rotate(time_t now)
{
	struct tm *tm = localtime(&now);
	char *fname;

	fname = build_str("%s" SPYLOG_FILE_NAME ".%04d%02d%02d",
			fprefix, 1900+tm->tm_year, tm->tm_mon, tm->tm_mday);

	if (spylog) fclose(spylog);
	spylog = fopen(fname, "a");
	if (!spylog)
	{
		fprintf(stderr, "can't append to %s\n", fname);
		freakout("fatal log rotation error");
	}
	free(fname);

	tm->tm_sec = tm->tm_min = tm->tm_hour = 0;
	next_rotate = mktime(tm) + 86400; /* tomorrow */
}



#define DISTANCE 12

void spylog_packet(byte *packet, word length,
		dword srcip, dword destip, word destport)
{
	char method[6], *url, *host = NULL;
	int start = 0, end, found = 0;
	time_t now;
	struct tm *tm;

	if (length < 6) return; /* strlen("get /") */

	/*{int i;for(i=0;i<length;i++)if(isalnum(packet[i]))putchar(packet[i]
	  );else putchar('.');putchar(10);return;}*/

	/* it has to be HTTP, allow for <DISTANCE> bytes of garbage */
	while (start + 5 < length && start <= DISTANCE && !found)
	{
		if (strncasecmp(packet+start,  "GET ", 4) == 0 ||
		    strncasecmp(packet+start, "HEAD ", 5) == 0 ||
		    strncasecmp(packet+start, "POST ", 5) == 0)
			found = 1;
		else
			start++;
	}

	if (!found) return;

	/* -- find method */
	end = start;
	while (end < length && packet[end] != ' ') end++;
	memcpy(method, packet+start, end-start);
	method[end-start] = 0;

	/* -- find url */
	start = ++end;
	while (end < length && packet[end] != ' '
			    && packet[end] != '\r'
			    && packet[end] != '\n') end++;
	url = (char*)malloc(end - start + 1);
	memcpy(url, packet+start, end-start);
	url[end-start] = 0;

	/* -- find host (optional) */
	start = end+1;
	while (start + 6 < length && memcmp(packet+start, "Host: ", 6) != 0)
		start++;

	if (start + 6 < length)
	{
		/* it exists - so extract it */
		start += 6;
		end = start;
		while (end<length && packet[end] != '\r'
				  && packet[end] != '\n') end++;

		host = (char*)malloc(end - start + 1);
		memcpy(host, packet+start, end-start);
		host[end-start] = 0;
	}
	else if (destport == 80)
		host = build_str("%d.%d.%d.%d",
				(destip >> 24) & 255,
				(destip >> 16) & 255,
				(destip >>  8) & 255,
				(destip      ) & 255);
	else
		host = build_str("%d.%d.%d.%d:%d",
				(destip >> 24) & 255,
				(destip >> 16) & 255,
				(destip >>  8) & 255,
				(destip      ) & 255,
				destport);

	now = time(NULL);
	if (now > next_rotate) spylog_rotate(now);
	tm = localtime(&now);

	fprintf(spylog, "%04d-%02d-%02d %02d:%02d:%02d %d.%d.%d.%d %s "
			"http://%s%s\n",

			1900+tm->tm_year, tm->tm_mon, tm->tm_mday,
			tm->tm_hour, tm->tm_min, tm->tm_sec,

			(srcip >> 24) & 255,
			(srcip >> 16) & 255,
			(srcip >>  8) & 255,
			(srcip      ) & 255,

			method, host, url);
	fflush(spylog);

	free(url);
	free(host);
}

